Stan Math Library: stan/math/prim/scal/prob/chi_square_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

chi_square_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_CHI_SQUARE_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_CHI_SQUARE_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/gamma_p.hpp>
15 #include <stan/math/prim/scal/fun/digamma.hpp>
16 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
17 #include <stan/math/prim/scal/fun/grad_reg_inc_gamma.hpp>
18 #include <stan/math/prim/scal/meta/include_summand.hpp>
19 #include <boost/random/chi_squared_distribution.hpp>
20 #include <boost/random/variate_generator.hpp>
21 #include <cmath>
22
23 namespace stan {
24
25 namespace math {
26
46 template <bool propto,
47 typename T_y, typename T_dof>
48 typename return_type<T_y, T_dof>::type
49 chi_square_log(const T_y& y, const T_dof& nu) {
50 static const char* function("stan::math::chi_square_log");
51 typedef typename stan::partials_return_type<T_y, T_dof>::type
52 T_partials_return;
53
54 // check if any vectors are zero length
55 if (!(stan::length(y)
56 && stan::length(nu)))
57 return 0.0;
58
59 using stan::math::check_positive_finite;
60 using stan::math::check_nonnegative;
61 using stan::math::check_not_nan;
62 using stan::math::check_consistent_sizes;
63 using stan::math::value_of;
64
65 T_partials_return logp(0.0);
66 check_not_nan(function, "Random variable", y);
67 check_nonnegative(function, "Random variable", y);
68 check_positive_finite(function, "Degrees of freedom parameter", nu);
69 check_consistent_sizes(function,
70 "Random variable", y,
71 "Degrees of freedom parameter", nu);
72
73
74 // set up template expressions wrapping scalars into vector views
75 VectorView<const T_y> y_vec(y);
76 VectorView<const T_dof> nu_vec(nu);
77 size_t N = max_size(y, nu);
78
79 for (size_t n = 0; n < length(y); n++)
80 if (value_of(y_vec[n]) < 0)
81 return LOG_ZERO;
82
83 // check if no variables are involved and prop-to
84 if (!include_summand<propto, T_y, T_dof>::value)
85 return 0.0;
86
87 using boost::math::digamma;
88 using boost::math::lgamma;
89 using stan::math::multiply_log;
90 using std::log;
91
92 VectorBuilder<include_summand<propto, T_y, T_dof>::value,
93 T_partials_return, T_y> log_y(length(y));
94 for (size_t i = 0; i < length(y); i++)
95 if (include_summand<propto, T_y, T_dof>::value)
96 log_y[i] = log(value_of(y_vec[i]));
97
98 VectorBuilder<include_summand<propto, T_y>::value,
99 T_partials_return, T_y> inv_y(length(y));
100 for (size_t i = 0; i < length(y); i++)
101 if (include_summand<propto, T_y>::value)
102 inv_y[i] = 1.0 / value_of(y_vec[i]);
103
104 VectorBuilder<include_summand<propto, T_dof>::value,
105 T_partials_return, T_dof> lgamma_half_nu(length(nu));
106 VectorBuilder<!is_constant_struct<T_dof>::value,
107 T_partials_return, T_dof>
108 digamma_half_nu_over_two(length(nu));
109
110 for (size_t i = 0; i < length(nu); i++) {
111 T_partials_return half_nu = 0.5 * value_of(nu_vec[i]);
112 if (include_summand<propto, T_dof>::value)
113 lgamma_half_nu[i] = lgamma(half_nu);
114 if (!is_constant_struct<T_dof>::value)
115 digamma_half_nu_over_two[i] = digamma(half_nu) * 0.5;
116 }
117
118 OperandsAndPartials<T_y, T_dof> operands_and_partials(y, nu);
119
120 for (size_t n = 0; n < N; n++) {
121 const T_partials_return y_dbl = value_of(y_vec[n]);
122 const T_partials_return half_y = 0.5 * y_dbl;
123 const T_partials_return nu_dbl = value_of(nu_vec[n]);
124 const T_partials_return half_nu = 0.5 * nu_dbl;
125 if (include_summand<propto, T_dof>::value)
126 logp += nu_dbl * NEG_LOG_TWO_OVER_TWO - lgamma_half_nu[n];
127 if (include_summand<propto, T_y, T_dof>::value)
128 logp += (half_nu-1.0) * log_y[n];
129 if (include_summand<propto, T_y>::value)
130 logp -= half_y;
131
132 if (!is_constant_struct<T_y>::value) {
133 operands_and_partials.d_x1[n] += (half_nu-1.0)*inv_y[n] - 0.5;
134 }
135 if (!is_constant_struct<T_dof>::value) {
136 operands_and_partials.d_x2[n] += NEG_LOG_TWO_OVER_TWO
137 - digamma_half_nu_over_two[n] + log_y[n]*0.5;
138 }
139 }
140 return operands_and_partials.value(logp);
141 }
142
143 template <typename T_y, typename T_dof>
144 inline
145 typename return_type<T_y, T_dof>::type
146 chi_square_log(const T_y& y, const T_dof& nu) {
147 return chi_square_log<false>(y, nu);
148 }
149
150 }
151 }
152 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::chi_square_log
return_type< T_y, T_dof >::type chi_square_log(const T_y &y, const T_dof &nu)
The log of a chi-squared density for y with the specified degrees of freedom parameter.
Definition: chi_square_log.hpp:49
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
gamma_p.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
grad_reg_inc_gamma.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::NEG_LOG_TWO_OVER_TWO
const double NEG_LOG_TWO_OVER_TWO
Definition: constants.hpp:191
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.